#if !defined(AFX_BT_PEER_LINK_H__50DC0701_DE7E_4A9D_B266_8C3188C24072__INCLUDED_)
#define AFX_BT_PEER_LINK_H__50DC0701_DE7E_4A9D_B266_8C3188C24072__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "alerts.h"
#include "bt_logger.h"
#include "bt_peer_data.h"
#include "bt_pl_write_data.h"
#include "data_counter.h"
#include "ring_buffer.h"
#include "socket.h"
#include "stream_writer.h"

class Cbt_file;
class Cbt_piece;
class Cserver;

class Cbt_peer_link: public Cbt_peer_data
{
public:
	Cserver* server();
	const Cserver* server() const;
	int time() const;
	string debug_string() const;
	int write_data(__int64 o, const char* s, int cb_s, int latency);
	int c_max_requests_pending() const;
	void check_pieces();
	Cbt_logger& logger();
	void clear_local_requests();
	int cb_write_buffer() const;
	void alert(Calert::t_level, const string&);
	int pre_dump() const;
	void dump(Cstream_writer&) const;
	void write_piece(int, int, int, const void*);
	void write_merkle_piece(__int64 offset, int size, const void* s, const string& hashes);
	void queue_have(int);
	void write_have(int);
	void write_haves();
	int read_handshake(const char* r);
	int read_message(const char* s, const char* s_end);
	void read_info(const char* r, const char* r_end);
	void write_keepalive();
	int read_piece(int, int, int, const char*);
	void read_merkle_piece(__int64 offset, int size, const char* s, const string& hashes);
	void write_handshake();
	void write_request(int, int, int);
	void write_merkle_cancel(__int64 offset);
	void write_merkle_request(__int64 offset, int c_hashes);
	void write_cancel(int, int, int);
	void write_get_info();
	void write_info();
	void write_get_peers();
	void write_peers();
	void choked(bool);
	void interested(bool);
	static byte* write16(byte* w, int v);
	static byte* write(byte* w, int v);
	void write_bitfield();
	void remote_requests(int, int, int);
	void remote_has(int);
	void remote_cancels(int, int, int);
	void remote_merkle_cancels(__int64 offset);
	void remote_merkle_requests(__int64 offset, int c_hashes);
	int send(int& send_quota);
	int recv();
	void write(const Cvirtual_binary&);
	void write(const void* s, int cb_s);
	void close();
	int pre_select(fd_set* fd_read_set, fd_set* fd_write_set, fd_set* fd_except_set);
	int post_select(fd_set* fd_read_set, fd_set* fd_write_set, fd_set* fd_except_set);
	Cbt_peer_link();
	~Cbt_peer_link();

	operator bool() const
	{
		return m_s != INVALID_SOCKET;
	}

	struct t_local_request
	{
		__int64 offset;
		int size;
		int stime;

		t_local_request()
		{
		}

		t_local_request(__int64 _offset, int _size, int _time)
		{
			offset = _offset;
			size = _size;
			stime = _time;
		}
	};

	struct t_remote_request
	{
		int c_hashes;
		__int64 offset;
		int size;

		t_remote_request()
		{
		}

		t_remote_request(__int64 _offset, int _size, int _c_hashes)
		{
			c_hashes = _c_hashes;
			offset = _offset;
			size = _size;
		}
	};

	typedef list<t_local_request> t_local_requests;
	typedef set<int> t_have_queue;
	typedef vector<bool> t_remote_pieces;
	typedef list<t_remote_request> t_remote_requests;
	typedef list<Cbt_pl_write_data> t_write_buffer;
	
	sockaddr_in m_a;
	Cbt_file* m_f;
	Csocket m_s;
	int m_state;
	Cring_buffer m_read_b;
	t_write_buffer m_write_b;
	t_have_queue m_have_queue;
	int m_rtime;
	int m_stime;
	int m_check_pieces_time;
	int mc_max_requests_pending;

	t_local_requests m_local_requests;
	int mc_local_requests_pending;
	t_remote_pieces m_remote_pieces;
	t_remote_requests m_remote_requests;
	Cdata_counter m_down_counter;
	Cdata_counter m_up_counter;
	int m_get_peers_stime;
	int m_peers_stime;
	bool m_get_info_extension;
	bool m_get_peers_extension;
	bool m_local_choked_goal;
	bool m_local_interested_goal;
	bool m_can_recv;
	bool m_can_send;
private:
	void write_choke(bool);
	void write_interested(bool);
};

#endif // !defined(AFX_BT_PEER_LINK_H__50DC0701_DE7E_4A9D_B266_8C3188C24072__INCLUDED_)
