<?php
include_once('File/Bittorrent/Decode.php');

class XBTException extends Exception {
	
}

class XBTIOException extends XBTException {
	public function __construct($message, $code=0){
		$this->message=$message;
		$this->code=$code;
	}	
}
class XBTInvalidArgumentsException extends XBTException {
	
}

class XBTClient{
	
	/**
	 * The Admin Port
	 * The port used for administrative interface
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_ADMIN_PORT = 'admin port';
	
	/**
	 * Path, where the completed torrents will be stored
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_COMPLETES_DIR = 'completes dir';
	
	/**
	 * Path where the incomplete torrents will be stored
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_INCOMPLETES_DIR = 'incompletes dir';
	
	/**
	 * Maximum number of peers the client will create (0=unlimited)
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_PEER_LIMIT = 'peer limit';
	
	/**
	 * The port to use for the peer2peer connections
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_PEER_PORT = 'peer port';
	
	/**
	 * The ratio between the total upload for a torrent and the torrent size
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_SEEDING_RATIO = 'seeding ratio';
	
	/**
	 * Maximum number of opened torrents
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_TORRENT_LIMIT = 'torrent limit';
	
	/**
	 * Path, where the .torrent files will be stored
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_TORRENTS_DIR = 'torrents dir';
	
	/**
	 * The UDP port of the tracker
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_TRACKER_PORT = 'tracker port';
	
	/**
	 * Maximum upload rate (in bytes/s)
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_UPLOAD_RATE = 'upload rate';
	
	/**
	 * Maximum allowed upload slots (0=unlimited)
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_UPLOAD_SLOTS = 'upload slots';
	
	/**
	 * Forced user agent on the backend
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_USER_AGENT = 'user agent';
	
	/**
	 * The firewall definitions list
	 * 
	 * @var string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:11:45 EET 2005
	 */
	const OPT_IP_LIST = 'ip list';
	
	const STATE_QUEUED = 0;
	const STATE_HASHING = 1;
	const STATE_STARTED = 2;
	const STATE_PAUSED = 3;
	const STATE_STOPPED = 4;
	
	const PRIORITY_HIGH = 1;
	const PRIORITY_NORMAL = 0;
	const PRIORITY_LOW = -1;
	
	const TORR_UPLOAD_RATE = 'up rate';
	const TORR_DOWNLOAD_RATE = 'down rate';
	const TORR_SEEDERS = 'complete';
	const TORR_LEECHERS = 'incomplete';
	const TORR_SEEDERS_TOTAL = 'complete total';
	const TORR_LEECHERS_TOTAL = 'incomplete total';
	const TORR_LEFT = 'left';
	const TORR_NAME = 'name';
	const TORR_PRIORITY = 'priority';
	const TORR_SIZE = 'size';
	const TORR_STATE = 'state';
	const TORR_DOWNLOADED = 'total downloaded';
	const TORR_UPLOADED = 'total uploaded';
	const TORR_EVENTS = 'events';
	
	
	private $server;
	private $port;
	private $socket;
	private $isConnected;
	
	private $options;
	private $status;
	
	/**
	 * BDecoder
	 * 
	 * @var File_Bittorrent_Decode
	 * @access private
	 * @author SASh
	 * @version 1.0 Tue Dec 13 17:36:33 EET 2005
	 */
	private $bdecoder;
	
	/**
	 * BEncoder
	 * 
	 * @var File_Bittorrent_Encode
	 * @access private
	 * @author SASh
	 * @version 1.0 Tue Dec 13 17:36:33 EET 2005
	 */
	private $bencoder;
	
	public $debug = false;
	
	public function __construct($server = 'localhost', $port = 6879){
		$this->server = $server;
		$this->port = $port;
		$this->bdecoder = new File_Bittorrent_Decode();
		$this->bencoder = new File_Bittorrent_Encode();
		
		$this->isConnected = false;
	}
	
	public function __destruct(){
		$this->close();
	}
	
	final static private function _funcHasSignature($args, $requirements){
		if (count($args) != count($requirements)) return false;
		foreach ($requirements as $id => $type){
			if ($type == 'mixed') continue;
			
			$arg_type = gettype($args[$id]);
			if ($arg_type == 'object'){
				if (!($args[$id] instanceof $type)){
					return false;
				}
			}
			else{
				if ($type != $arg_type) return false;
			}
		}
		return true;
	}
	
	private function open(){
		$this->socket = @fsockopen($this->server, $this->port, $errorno, $errorstr, 5);
		if (!$this->socket){
			throw new XBTIOException($errorstr, $errorno); 
		}
		$this->isConnected = true;
	}
	
	private function close(){
		if ($this->isConnected){
			return fclose($this->socket);
		}
		return false;
	}

	public function setServer($server){
		$this->server = $server;
	}
	
	public function setPort($port){
		$this->port = $port;
	}
	
	public function getServer(){
		return $this->server;
	}
	
	public function getPort(){
		return $this->port;
	}

	/**
	 * Send a command to the server and return the responce
	 * 
	 * @return mixed
	 * @param mixed $cmd
	 * @access protected
	 * @throws XBTIOException On connection failure
	 * @throws XBTInvalidArgumentsException 
	 * @author SASh
	 * @version 1.0 Tue Dec 13 17:41:04 EET 2005
	 */
	protected function sendCommand($cmd){
		if (!$this->isConnected){
			$this->open();
		}
		if ($this->debug){
			echo "-----\nXBTClient: Sending command\n";
			print_r($cmd);
			echo "\n";
		}
		$cmd = $this->bencoder->encode($cmd);
		if (is_null($cmd)){
			throw new XBTInvalidArgumentsException("Unable to encode the message");
		}
		
		$cmd = pack('N', strlen($cmd) + 1) . chr(0x40) . $cmd;
		if (fwrite($this->socket, $cmd) != strlen($cmd)){
			throw new XBTIOException("Communication with the server failed ('fwrite failed')");
		}
			
		$d = '';
		while (strlen($d) < 4){
			$r = fread($this->socket, 4 - strlen($d));
			if (!strlen($r)){
				throw new XBTIOException("Less then 4 bytes read!");
			}
			$d .= $r;
		}
		$l = unpack('N', $d);
		$l = $l[1];
		$d = '';
		while (strlen($d) < $l){
			$r = fread($this->socket, $l - strlen($d));
			if (!strlen($r)){
				throw new XBTIOException("The entire message not read!");
			}
			$d .= $r;
		}
		
		if ($this->debug){
			echo "XBTClient: Response RAW\n";
			print_r(substr($d, 1));
			echo "\n\n";
		}
		
		$res = $this->bdecoder->decode(substr($d, 1));
		if ($res === false){
			throw new XBTIOException("Unable to decode the responce", substr($d, 1));
		}
		if ($this->debug){
			echo "XBTClient: Response\n";
			print_r($res);
			echo "\n-----\n\n";
		}
		return $res;
	}
	
	public function rawCommand($cmd){
		return $this->sendCommand($cmd);
	}
	
	/**
	 * Refresh the status of the backend
	 * Make a new request and updates the status
	 * 
	 * @return void
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 15 12:43:16 EET 2005
	 */
	public function refreshStatus(){
		$this->status = $this->sendCommand(array('action'=>'get status'));
	}
	
	/**
	 * Get the backend version
	 * 
	 * @return string
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 14 12:37:13 EET 2005
	 */
	public function getVersion(){
		if (!is_array($this->status)){
			$this->refreshStatus();
		}
		return $this->status['version'];
	}
	
	/**
	 * Fetch an array with standart information for a torrent (based on the get status command)
	 * 
	 * @return array|mixed
	 * @param string $infohash
	 * @param string $key The resuired specific key (of not set, the entire array with all keys will be returned)
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:23:33 EET 2005
	 */
	public function getTorrentInfo($infohash, $key = null){
		if (!is_array($this->status)){
			$this->refreshStatus();
		}
		$infohash = pack('H*', $infohash);
		return is_null($key) ? $this->status['files'][$infohash] : $this->status['files'][$infohash][$key];
	}
	
	/**
	 * Fetch an array with standart information for a torrent (based on the get status command)
	 * Based on the local torrent file
	 * 
	 * @return array
	 * @param string $filepath
	 * @param string $key The resuired specific key (of not set, the entire array with all keys will be returned)
	 * @access public
	 * @see XBTClient::getTorrentInfo()
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:23:33 EET 2005
	 */
	public function getTorrentInfo_FromFile($filepath, $key = null){
		if ($this->bdecoder->decodeFile($filepath)){
			return $this->getTorrentInfo($this->bdecoder->info_hash, $key);
		}
		else{
			throw new XBTInvalidArgumentsException('Unable to decode the torrent file');
		}
	}
	
	/**
	 * Get the list of all torrents info hashesh, in the backend
	 * 
	 * @return array Infohashes list
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 29 16:02:37 EET 2005
	 */
	public function listTorrents(){
		if (!is_array($this->status)){
			$this->refreshStatus();
		}
		$res = array();
		foreach ($this->status['files'] as $infohash=>$null){
			$un = @unpack('H*', $infohash);
			$res[] = $un[1];
		}
		return $res;
	}
	
	/**
	 * Get the count of all torrents
	 * The torrents could be in any state (started, queued, stopped, and so on)
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 29 16:16:46 EET 2005
	 */
	public function countTorrents(){
		return count($this->listTorrents());
	}
	
	/**
	 * Get the infohash list of all running torrents in seeding state
	 * 
	 * @return array
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 29 16:16:46 EET 2005
	 */
	public function listSeedingTorrents(){
		$res = array();
		foreach ($this->listTorrents() as $infohash){
			if (
				$this->getTorrentInfo($infohash, self::TORR_STATE) == self::STATE_STARTED 
				&& $this->getTorrentInfo($infohash, self::TORR_LEFT) === 0
			){
				$res[] = $infohash;
			}
		}
		return $res;
	}
	
	/**
	 * Get the infohash list of all running torrents in leeching state
	 * 
	 * @return array
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 29 16:16:46 EET 2005
	 */
	public function listLeechingTorrents(){
		$res = array();
		foreach ($this->listTorrents() as $infohash){
			if (
				$this->getTorrentInfo($infohash, self::TORR_STATE) == self::STATE_STARTED 
				&& $this->getTorrentInfo($infohash, self::TORR_LEFT) !== 0
			){
				$res[] = $infohash;
			}
		}
		return $res;
	}
	
	public function listTorrentsInState($state){
		$res = array();
		foreach ($this->listTorrents() as $infohash){
			if ($this->getTorrentInfo($infohash, self::TORR_STATE) == $state){
				$res[] = $infohash;
			}
		}
		return $res;
	}
	
	/**
	 * Get the count of all running torrents in seeding state
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 29 16:16:46 EET 2005
	 */
	public function countSeedingTorrents(){
		return count($this->listSeedingTorrents());
	}
	
	/**
	 * Get the count of all running torrents in seeding state
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 29 16:16:46 EET 2005
	 */
	public function countLeechingTorrents(){
		return count($this->listLeechingTorrents());
	}
	
	/**
	 * Sum the given $key value for every known file
	 * 
	 * @return int
	 * @param string $key On of the XBTClient::TORR_* constants
	 * @access protected
	 * @author SASh
	 * @version 1.0 Thu Dec 29 16:22:54 EET 2005
	 */
	protected function sumFilesStatus($key){
		if (!is_array($this->status)){
			$this->refreshStatus();
		}
		$res = 0;
		foreach ($this->status['files'] as $file){
			$res += $file[$key];
		}
		return $res;
	}
	
	/**
	 * Get the total upload speed of the backend (in bytes/second)
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Fri Dec 23 15:09:53 EET 2005
	 */
	public function getUploadRate(){
		return $this->sumFilesStatus(self::TORR_UPLOAD_RATE);
	}
	
	/**
	 * Get the total download speed of the backend (in bytes/second)
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Fri Dec 23 15:09:53 EET 2005
	 */
	public function getDownloadRate(){
		return $this->sumFilesStatus(self::TORR_DOWNLOAD_RATE);
	}
	
	/**
	 * Get the total number of seeders, connected to the backend
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Fri Dec 23 15:09:53 EET 2005
	 */
	public function countSeeders(){
		return $this->sumFilesStatus(self::TORR_SEEDERS);
	}
	
	/**
	 * Get the total number of leechers, connected to the backend
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Fri Dec 23 15:09:53 EET 2005
	 */
	public function countLeechers(){
		return $this->sumFilesStatus(self::TORR_LEECHERS);
	}
	
	/**
	 * Get the total number of peers (seeders+leechers), connected to the backend
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Fri Dec 23 15:09:53 EET 2005
	 */
	public function countPeers(){
		return $this->countLeechers() + $this->countSeeders();
	}
	
	/**
	 * The total bytes left to download
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 29 16:40:04 EET 2005
	 */
	public function getTotalLeft(){
		return $this->sumFilesStatus(self::TORR_LEFT);
	}
	
	/**
	 * The total bytes left to download
	 * 
	 * @return int
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 29 16:40:04 EET 2005
	 */
	public function getTotalSize(){
		return $this->sumFilesStatus(self::TORR_SIZE);
	}
	
	/**
	 * Refresh the options settings of the backend
	 * Make a new request and update all settings
	 * 
	 * @return void
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 15 12:40:57 EET 2005
	 */
	public function refreshOptions(){
		$this->options = $this->sendCommand(array('action' => 'get options'));
	}
	
	/**
	 * Get an options settings from the backend
	 * Returns the requested option, ot an array with a list of options,
	 * if more than one option is requested (format: XBTClient::OPT_* => value)
	 * 
	 * @return mixed
	 * @param string|array $option One of the XBTClient::OPT_* constants
	 * @access public
	 * @throws XBTInvalidArgumentsException on invalid option key given
	 * @author SASh
	 * @version 1.0 Thu Dec 15 12:39:50 EET 2005
	 */
	public function getOption($option){
		$args = func_get_args();
		$options=array();
		if (self::_funcHasSignature($args, array('array'))){
			list($options) = $args;
		}
		elseif (self::_funcHasSignature($args, array('string'))){
			list($option) = $args;
			$options[] = $option;
		}
		else{
			throw new XBTInvalidArgumentsException('getOption(): Unrecognized function signature');
		}
	
		if (!is_array($this->options)){
			$this->refreshOptions();
		}
		$res = array();
		foreach ($options as $opt){
			if (isset($this->options[$opt])){
				$res[$opt] = $this->options[$opt];
			}
			else{
				throw new XBTInvalidArgumentsException('Options key "'.$opt.'" not found!');
			}
		}
		
		return count($res) == 1 ? array_pop($res) : $res;
	}
	
	/**
	 * Set an options for the backend
	 * The overload of the method, accepting one array, will set all options 
	 * listet in that array simultaniously
	 * The array has to have the fallowing format: XBTClient::OPT_* => value
	 * 
	 * @return int Always returns 0!
	 * @param string|array $option One of the XBTClient::OPT_* constants
	 * @param mixed $value
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 15 12:39:50 EET 2005
	 */
	public function setOption($option, $value){
		$args = func_get_args();
		$options=array();
		if (self::_funcHasSignature($args, array('array'))){
			list($options) = $args;
		}
		elseif (self::_funcHasSignature($args, array('string', 'mixed'))){
			list($option, $value) = $args;
			$options[$option] = $value;
		}
		else{
			throw new XBTInvalidArgumentsException('setOption(): Unrecognized function signature');
		}
		
		$options['action'] = 'set options';
		return $this->sendCommand(
			$options
		);
	}
	
	/**
	 * Open a torrent file for leeching/seeding
	 * Returns 0 on OK and >0 on Error
	 * 
	 * @return int
	 * @param string $torrent_contents The contents of the .torrent file
	 * @access public
	 * @author SASh
	 * @version 1.0 Thu Dec 15 13:55:51 EET 2005
	 */
	public function openTorrent($torrent_contents){
		if (!is_string($torrent_contents)){
			throw new XBTInvalidArgumentsException('Invalid input parameter (string needed)');
		}
		$resp = $this->sendCommand(
			array(
				'action' => 'open torrent',
				'torrent' => $torrent_contents
			)
		);
		
		return $resp;
	}
	
	/**
	 * Wrapper of the openTorrent function that uses a filepath to open the torrent
	 * Returns 0 on OK and >0 on Error
	 * 
	 * @return int
	 * @param string $torrent_filepath
	 * @access public
	 * @see XBTClient::openTorrent()
	 * @author SASh
	 * @version 1.0 Thu Dec 15 13:56:47 EET 2005
	 */
	public function openTorrent_FromFile($torrent_filepath){
		return $this->openTorrent((string)@file_get_contents($torrent_filepath));
	}
	
	/**
	 * Close a torrent file and stop leeching/seeding
	 * Returns 0 on OK and >0 on Error
	 * 
	 * @return int
	 * @param string $hash
	 * @param bool $remove_torrent
	 * @param bool $remove_data
	 * @access public
	 * @see XBTClient::sendCommand()
	 * @author SASh
	 * @version 1.0 Thu Dec 15 15:14:51 EET 2005
	 */
	public function closeTorrent($hash, $remove_torrent = false, $remove_data = false){
		$hash = pack('H*', $hash);
		return $this->sendCommand(array('action' => 'close torrent', 'hash' => $hash, 'remove torrent' => (int)$remove_torrent, 'remove data' => (int)$remove_data));
	}
	
	/**
	 * Close a torrent file and stop leeching/seeding
	 * Returns 0 on OK and >0 on Error
	 * 
	 * @return int
	 * @param string $filepath
	 * @param bool $remove_torrent
	 * @param bool $remove_data
	 * @access public
	 * @see XBTClient::closeTorrent()
	 * @author SASh
	 * @version 1.0 Thu Dec 15 15:14:51 EET 2005
	 */
	public function closeTorrent_FromFile($filepath, $remove_torrent = false, $remove_data = false){
		if ($this->bdecoder->decodeFile($filepath)){
			return $this->closeTorrent($this->bdecoder->info_hash, $remove_torrent, $remove_data);
		}
		else{
			throw new XBTInvalidArgumentsException('Unable to decode the torrent file');
		}
	}
	
	/**
	 * Set the torrent state
	 * Returns 0 on OK and >0 on Error
	 * 
	 * @return int
	 * @param int $state One of the XBTClient::STATE_* constants
	 * @param string $infohash
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:26:56 EET 2005
	 */
	public function setTorrentState($state, $infohash){
		$infohash = pack('H*', $infohash);
		return $this->sendCommand(array('action' => 'set state', 'hash' => $infohash, 'state' => $state));
	}
	
	/**
	 * Set the torrent state
	 * Returns 0 on OK and >0 on Error
	 * 
	 * @return int
	 * @param int $state One of the XBTClient::STATE_* constants
	 * @param string $filepath
	 * @see XBTClient::setTorrentState()
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:26:56 EET 2005
	 */
	public function setTorrentState_FromFile($state, $filepath){
		if ($this->bdecoder->decodeFile($filepath)){
			return $this->setTorrentState($state, $this->bdecoder->info_hash);
		}
		else{
			throw new XBTInvalidArgumentsException('Unable to decode the torrent file');
		}
	}
	
	/**
	 * Set the torrent priority
	 * Returns 0 on OK and >0 on Error
	 * 
	 * @return int
	 * @param int $priority One of the XBTClient::PRIORITY_* constants
	 * @param string $infohash
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:26:56 EET 2005
	 */
	public function setTorrentPriority($priority, $infohash){
		$infohash = pack('H*', $infohash);
		return $this->sendCommand(array('action' => 'set priority', 'hash' => $infohash, 'priority' => $priority));
	}
	
	/**
	 * Set the torrent priority
	 * Returns 0 on OK and >0 on Error
	 * 
	 * @return int
	 * @param int $priority One of the XBTClient::PRIORITY_* constants
	 * @param string $filepath
	 * @see XBTClient::setTorrentState()
	 * @access public
	 * @author SASh
	 * @version 1.0 Wed Dec 21 19:26:56 EET 2005
	 */
	public function setTorrentPriority_FromFile($priority, $filepath){
		if ($this->bdecoder->decodeFile($filepath)){
			return $this->setTorrentPriority($priority, $this->bdecoder->info_hash);
		}
		else{
			throw new XBTInvalidArgumentsException('Unable to decode the torrent file');
		}
	}
	
	public static function hasError($response){
		if (!is_array($response)) return false;
		foreach ($response as $item){
			if (!is_array($item)) return false;
			if (isset($item['code']) && isset($item['error'])){
				return true;
			}
		}
		return false;
	}
	
}
?>