#!/usr/bin/php5
<?php
/**
 * Example use:
 * 
 * xbt_cli.php "array('action'=>'get options')"
 * // List the backend options
 * 
 * xbt_cli.php "array('action'=>'set state', 'hash'=>pack('H*', '0d1ffbba17b2d62f68f2bb9a4bef7c9d2af5fdce'), 'state'=>4)"
 * // Stops the torrent with info hash 0d1ffbba17b2d62f68f2bb9a4bef7c9d2af5fdce
 * 
 */

require_once('XBTClient.php');

if (@include_once('Console/Getargs.php')){
	define('CONSOLE_GETARGS', true);
}
else{
	define('CONSOLE_GETARGS', false);
}




function mkrate($bps){
	if ($bps < 1024){
		return $bps.' b';
	}
	if ($bps < 1024*1024){
		return number_format($bps/1024, 1).' Kb';
	}
	if ($bps < 1024*1024*1024){
		return number_format($bps/(1024*1024), 1).' Mb';
	}
	if ($bps < 1024*1024*1024*1024){
		return number_format($bps/(1024*1024*1024), 1).' Gb';
	}
}

// Input fetching
if (CONSOLE_GETARGS){
	$console_args_conf = array(
		'server' => array(
			'short' => 's',
			'min' => 1,
			'max' => 1,
			'desc' => 'The IP of the backend',
			'default' => '127.0.0.1'
		),
		'raw' => array(
			'short' => 'r',
			'min' => 1,
			'max' => 1,
			'desc' => 'The raw command to execute to the backend',
		),
		'action' => array(
			'short' => 'a',
			'min' => 1,
			'max' => 1,
			'desc' => 'The action to send',
			//'default' => 'get options',
		),
		'extra' => array(
			'short' => 'x',
			'min' => 1,
			'max' => 1,
			'desc' => 'The additional action paramesers to send (Used with -a)',
		),
		'xbtclientcommand' => array(
			'short' => 'c',
			'min' => 1,
			'max' => 1,
			'desc' => 'Force a public function of the XBTClient class',
		),
		'xbtclientparams' => array(
			'short' => 'p',
			'min' => 1,
			'max' => 1,
			'desc' => 'The array of params to use with -c'
		),
	);
	
	//$args = new Console_Getargs_Options();
	
	$args = Console_Getargs::factory($console_args_conf);
	if (PEAR::isError($args)) {
		if ($args->getCode() === CONSOLE_GETARGS_ERROR_USER) {
			echo Console_Getargs::getHelp($console_args_conf, null, $args->getMessage() . "\n");
		} else if ($args->getCode() === CONSOLE_GETARGS_HELP) {
			echo "Usage: xbt_cli.php \n";
			echo "\t-s (--server) The IP of the backend (default 127.0.0.1)\n";
			echo "\t-r (--raw) The raw command to send to the backend (in php array definition)\n";
			echo "\t-a (--action) [ACTION] The action to send\n";
			echo "\t-x (--extra) The additional action paramesers to send (Used with -a)\n";
			echo "\t\t Example: xbt_cli.php -a 'close torrent' -x \"'hash'=>pack('H*', '212eaa3afd97cb5590bcce32dc717fa396c2c912'), 'remove torrent'=>1\" \n";
			echo "\n";
			echo "\t-c (--xbtclientcommand) [FNAME] Force a public function of the XBTClient class\n";
			echo "\t-p (--xbtclientparams) The array of params to use with -c\n";
			echo "\t\t Example: xbt_cli.php -c openTorrent_FromFile -p \"'/home/user/my_torrent.torrent'\" \n";
			echo "\t\t Example: xbt_cli.php -c setTorrentState -p \"4, '212eaa3afd97cb5590bcce32dc717fa396c2c912'\"; # stops the torrent\n";
			echo "\n";
			echo "\t FNAME: \n";
			$methods = get_class_methods('XBTClient');
			foreach ($methods as $method){
				echo "\t  $method \n";
			}
			echo "\n";
			echo "\t ACTION: \n";
			echo "\t  open torrent \n";
			echo "\t  close torrent \n";
			echo "\t  get status \n";
			echo "\t  set priority \n";
			echo "\t  set state \n";
			echo "\t  get options \n";
			echo "\t  set options \n";
			echo "\t Additional ACTIONS: \n";
			echo "\t  add net filter \n";
			echo "\t  remove net filter \n";
			
		}
		exit();
	}
	
	$ip = $args->getValue('server');
	
	if ($args->isDefined('raw')){
		$input = eval('return '.$args->getValue('raw').';');
	}
	
	// Additional options
	if ($args->isDefined('action')){
		$action = $args->getValue('action');
	}
	if ($args->isDefined('extra')){
		$action_params = eval('return array('.$args->getValue('extra').');');
	}
	else{
		$action_params = array();
	}
	
	if ($args->isDefined('xbtclientcommand')){
		$XBTClient_function = $args->getValue('xbtclientcommand');
	}
	if ($args->isDefined('xbtclientparams')){
		$XBTClient_params = eval('return array('.$args->getValue('xbtclientparams').');');
	}
	else{
		$XBTClient_params = array();
	}
	
}
else{
	if (in_array('--help', $argv) || in_array('-h', $argv)){
		echo "Usage: xbt_cli.php [backend-ip] [command]\n";
		exit();
	}
	
	if (count($argv) >= 3){
		$ip = $argv[1];
		$input = $argv[2];
	}
	else{
		if (count($argv) >= 2){ // One param only
			// Quess IP or command
			if (ip2long($argv[1]) === false){
				$input = $argv[1];
			}
			else{
				// goto demo
				$ip = $argv[1]; 
			}
		}
		if (!$ip){
			$ip = '127.0.0.1';
		}
	}
	$input = "return ".$input.";";

	$input = eval($input);
}
// End: Input fetching

$xbt = new XBTClient($ip);

// XBTClient command execution
if (isset($XBTClient_function)){
	
	if ( method_exists($xbt, $XBTClient_function)){
		echo "Executing to $ip XBTClient::$XBTClient_function \n";
		print_r($XBTClient_params);
		try{
			echo "Result: \n";
			print_r(call_user_func_array(array($xbt, $XBTClient_function), $XBTClient_params));
			echo "\n";
		}
		catch (Exception $e){
			echo "Error: ".$e->getMessage()."\n";
		}
	}
	else{
		echo "Error: Method $XBTClient_function is not defined in the XBTClient class! \n";
	}
}
// END: XBTClient command execution



// Exclusive command execution
if (isset($action)){
	$input = array_merge(array('action' => $action), $action_params);
}
// END: Exclusive command execution

if (isset($input)){
	echo "Sending to $ip: \n";
	print_r($input);
	try {
		echo "\nResponse: \n";
		print_r($xbt->rawCommand($input));
		echo "\n";
	}
	catch (Exception $e){
		echo "Error: ".$e->getMessage()."\n";
	}
}


try{
	$xbt->refreshStatus();
	$xbt->refreshOptions();
	echo "\nFull torrents list:\n";
	$cnt = 1;
	$list = $xbt->listTorrents();
	$list_len = count($list);
	$list_len_digits = strlen((string)$list_len);
	$columns_length = array($list_len_digits+2, 4, 3, 24, 14, 5, 5, 50);
	foreach ($list as $infohash){
		$perc = number_format((($xbt->getTorrentInfo($infohash, XBTClient::TORR_SIZE) - $xbt->getTorrentInfo($infohash, XBTClient::TORR_LEFT))/$xbt->getTorrentInfo($infohash, XBTClient::TORR_SIZE))*100, 2)."%";
		$ratio = ($xbt->getTorrentInfo($infohash, XBTClient::TORR_DOWNLOADED) ? number_format($xbt->getTorrentInfo($infohash, XBTClient::TORR_UPLOADED) / $xbt->getTorrentInfo($infohash, XBTClient::TORR_DOWNLOADED), 3) : 'app. '.number_format($xbt->getTorrentInfo($infohash, XBTClient::TORR_UPLOADED) / $xbt->getTorrentInfo($infohash, XBTClient::TORR_SIZE), 3))." ratio";
		$eta_sec = 0;
		$eta_min = 0;
		$eta_hour = 0;
		if ($xbt->getTorrentInfo($infohash, XBTClient::TORR_DOWNLOAD_RATE)){
			$eta_sec = $xbt->getTorrentInfo($infohash, XBTClient::TORR_LEFT)/$xbt->getTorrentInfo($infohash, XBTClient::TORR_DOWNLOAD_RATE);
			$eta_min = floor($eta_sec/60);
			$eta_sec %= 60;
			$eta_hour = floor($eta_min/60); //min
			$eta_min %= 60;
		}
		
		$eta = ($eta_hour ? $eta_hour . 'h ' :'').($eta_min ? $eta_min . 'm ' :'').$eta_sec.'s';
		
		switch ($xbt->getTorrentInfo($infohash, XBTClient::TORR_STATE)){
			case XBTClient::STATE_PAUSED:
				$state = "[P]";
				break;
			case XBTClient::STATE_QUEUED:
				$state = "[Q]";
				break;
			case XBTClient::STATE_STARTED:
				$state = "[R]";
				break;
			case XBTClient::STATE_STOPPED:
				$state = "[S]";
				break;
			case XBTClient::STATE_HASHING:
				$state = "[H]";
				break;
		}
		
		
		$columns = array(
			str_pad($cnt, $list_len_digits, 0, STR_PAD_LEFT).".",
			$state,
			$xbt->getTorrentInfo($infohash, XBTClient::TORR_LEFT) == 0 ? "->" : "<-",
			$xbt->getTorrentInfo($infohash, XBTClient::TORR_LEFT) == 0 ? $ratio:"$perc ETA: ".$eta,
			$xbt->getTorrentInfo($infohash, XBTClient::TORR_LEFT) == 0 ? '@ '.mkrate($xbt->getTorrentInfo($infohash, XBTClient::TORR_UPLOAD_RATE)).'/s':'@ '.mkrate($xbt->getTorrentInfo($infohash, XBTClient::TORR_DOWNLOAD_RATE)).'/s',
			"S:".$xbt->getTorrentInfo($infohash, XBTClient::TORR_SEEDERS),
			"L:".$xbt->getTorrentInfo($infohash, XBTClient::TORR_LEECHERS),
			
			basename($xbt->getTorrentInfo($infohash, XBTClient::TORR_NAME ))." ($infohash)",
		);
		
		foreach ($columns as $id=>$value){
			//echo ">".strlen($value);
			echo str_pad($value, $columns_length[$id], ' ', STR_PAD_RIGHT);
		}
		
		
		echo "\n";
		$cnt++;
	}
}
catch(Exception $e){
	echo "Fault getting data: ".$e->getMessage()."\n";
}
?>